/*************************************************************
 *
 *  MathJax/jax/output/HTML-CSS/entities/v.js
 *
 *  Copyright (c) 2010 Design Science, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

(function (MATHML) {
  MathJax.Hub.Insert(MATHML.Parse.Entity,{
    'VDash': '\u22AB',
    'Vbar': '\u2AEB',
    'Vcy': '\u0412',
    'Vdashl': '\u2AE6',
    'Verbar': '\u2016',
    'Vert': '\u2016',
    'VerticalLine': '\u007C',
    'VerticalSeparator': '\u2758',
    'VeryThinSpace': '\u200A',
    'vArr': '\u21D5',
    'vBar': '\u2AE8',
    'vBarv': '\u2AE9',
    'vDash': '\u22A8',
    'vangrt': '\u299C',
    'varepsilon': '\u03F5',
    'varkappa': '\u03F0',
    'varnothing': '\u2205',
    'varphi': '\u03D5',
    'varpi': '\u03D6',
    'varpropto': '\u221D',
    'varr': '\u2195',
    'varrho': '\u03F1',
    'varsigma': '\u03C2',
    'varsubsetneq': '\u228A\uFE00',
    'varsubsetneqq': '\u2ACB\uFE00',
    'varsupsetneq': '\u228B\uFE00',
    'varsupsetneqq': '\u2ACC\uFE00',
    'vartheta': '\u03D1',
    'vartriangleleft': '\u22B2',
    'vartriangleright': '\u22B3',
    'vcy': '\u0432',
    'vdash': '\u22A2',
    'vee': '\u2228',
    'veeeq': '\u225A',
    'verbar': '\u007C',
    'vert': '\u007C',
    'vltri': '\u22B2',
    'vnsub': '\u2282\u20D2',
    'vnsup': '\u2283\u20D2',
    'vprop': '\u221D',
    'vrtri': '\u22B3',
    'vsubnE': '\u2ACB\uFE00',
    'vsubne': '\u228A\uFE00',
    'vsupnE': '\u2ACC\uFE00',
    'vsupne': '\u228B\uFE00',
    'vzigzag': '\u299A'
  });

  MathJax.Ajax.loadComplete(MATHML.entityDir+"/v.js");

})(MathJax.InputJax.MathML);
