/*************************************************************
 *
 *  MathJax/jax/output/HTML-CSS/entities/m.js
 *
 *  Copyright (c) 2010 Design Science, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

(function (MATHML) {
  MathJax.Hub.Insert(MATHML.Parse.Entity,{
    'Map': '\u2905',
    'Mcy': '\u041C',
    'MediumSpace': '\u205F',
    'Mellintrf': '\u2133',
    'Mu': '\u039C',
    'mDDot': '\u223A',
    'male': '\u2642',
    'maltese': '\u2720',
    'map': '\u21A6',
    'mapsto': '\u21A6',
    'mapstodown': '\u21A7',
    'mapstoleft': '\u21A4',
    'mapstoup': '\u21A5',
    'marker': '\u25AE',
    'mcomma': '\u2A29',
    'mcy': '\u043C',
    'mdash': '\u2014',
    'measuredangle': '\u2221',
    'micro': '\u00B5',
    'mid': '\u2223',
    'midast': '\u002A',
    'midcir': '\u2AF0',
    'middot': '\u00B7',
    'minus': '\u2212',
    'minusb': '\u229F',
    'minusd': '\u2238',
    'minusdu': '\u2A2A',
    'mlcp': '\u2ADB',
    'mldr': '\u2026',
    'mnplus': '\u2213',
    'models': '\u22A7',
    'mp': '\u2213',
    'mstpos': '\u223E',
    'mumap': '\u22B8'
  });

  MathJax.Ajax.loadComplete(MATHML.entityDir+"/m.js");

})(MathJax.InputJax.MathML);
